@@ -13,14 +13,14 @@
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
-static char sccsid[] = "@(#) tkTextTag.c 1.28 95/05/25 10:36:17";
+static char sccsid[] = "@(#) tkTextTag.c 1.29 95/11/24 15:46:17";
 
 #include "default.h"
 #include "tkPort.h"
 #include "tk.h"
 #include "tkText.h"
 
-static char rcsid[] = "$Id: tkTextTag.c,v 1.6 1995/09/09 06:45:07 ilya Exp $";
+static char rcsid[] = "$Id: tkTextTag.c,v 1.7 1996/03/12 00:09:47 ilya Exp $";
 
 /*
  * Information used for parsing tag configuration information:
@@ -69,6 +69,9 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_UID, "-wrap",          NULL,          NULL,
 	         NULL, Tk_Offset(TkTextTag, wrapMode),
 	TK_CONFIG_NULL_OK},
+    {TK_CONFIG_LANGARG, "-data",          NULL,          NULL,
+	         NULL, Tk_Offset(TkTextTag, userData),
+	TK_CONFIG_NULL_OK},
     {TK_CONFIG_END,          NULL,          NULL,          NULL,
 	         NULL, 0, 0}
 };
@@ -1111,7 +1114,8 @@ TkTextBindProc(clientData, eventPtr)
 	}
 	TkTextPickCurrent(textPtr, eventPtr);
     }
-    if ((textPtr->numCurTags > 0) && (textPtr->bindingTable != NULL)) {
+    if ((textPtr->numCurTags > 0) && (textPtr->bindingTable != NULL)
+	    && (textPtr->tkwin != NULL)) {
 	Tk_BindEvent(textPtr->bindingTable, eventPtr, textPtr->tkwin,
 		textPtr->numCurTags, (ClientData *) textPtr->curTagArrayPtr);
     }
@@ -1146,7 +1150,9 @@ TkTextBindProc(clientData, eventPtr)
  * Side effects:
  *	The current mark for textPtr may change.  If it does,
  *	then the commands associated with character entry and leave
- *	could do just about anything.
+ *	could do just about anything.  For example, the text widget
+ *	might be deleted.  It is up to the caller to protect itself
+ *	with calls to Tk_Preserve and Tk_Release.
  *
  *--------------------------------------------------------------
  */
@@ -1281,7 +1287,7 @@ TkTextPickCurrent(textPtr, eventPtr)
     oldArrayPtr = textPtr->curTagArrayPtr;
     textPtr->curTagArrayPtr = newArrayPtr;
     if (numOldTags != 0) {
-	if (textPtr->bindingTable != NULL) {
+	if ((textPtr->bindingTable != NULL) && (textPtr->tkwin != NULL)) {
 	    event = textPtr->pickEvent;
 	    event.type = LeaveNotify;
 
@@ -1309,7 +1315,7 @@ TkTextPickCurrent(textPtr, eventPtr)
 	    textPtr->pickEvent.xcrossing.y, &index);
     TkTextSetMark(textPtr, "current", &index);
     if (numNewTags != 0) {
-	if (textPtr->bindingTable != NULL) {
+	if ((textPtr->bindingTable != NULL) && (textPtr->tkwin != NULL)) {
 	    event = textPtr->pickEvent;
 	    event.type = EnterNotify;
 	    event.xcrossing.detail = NotifyAncestor;
